(********************************************************************)
(*																	*)
(*						Basic Drawing Objects						*)
(*																	*)
(********************************************************************)

unit TClass;

interface

type TByteFile = File of Byte;

type TMap = array[0..31,0..23] of Byte;	{ Map array }

type TLevel = object
	LevelID:Byte;						{ Room number }
	Map:TMap;							{ Tile map }
	NewFile:Boolean;
	constructor Init;
	procedure	Load(ID:Integer);
	procedure	Save;
	function	FileName:String;
end;

type TTileGraphic = object				{ A Tile graphic }
	Pixels:array[0..7] of Byte;
	constructor	Init;
	procedure 	SetPixel(x,y:Integer;On:Boolean);
	function  	GetPixel(x,y:Integer):Boolean;
	procedure   Read(var f:TByteFile);
	procedure	Write(var f:TByteFile);
	procedure	Draw(x,y,Size,FCol:Integer);
end;

const	TileSetSize = 16;

type TTileSet = object					{ Set of 16 tiles }
	Tile:array[0..TileSetSize-1] of TTileGraphic;
	constructor Init;
	destructor	Done;
end;

type PLevel = ^TLevel;					{ Define pointers }
	 PTileGraphic = ^TTileGraphic;
     PTileSet = ^TTileSet;

implementation

uses Graph;

(************************************************************)
(*	Initialise the room object by setting everything to 0	*)
(************************************************************)

constructor TLevel.Init;
var x,y:Integer;
begin
	LevelID := 0;
	for x := 0 to 31 do
		begin
			for y := 0 to 23 do
				begin
					Map[x,y] := 0;
					Map[0,y] := 1;
					Map[31,y] := 1;
				end;
			Map[x,0] := 1;
			Map[x,23] := 1;
		end;
end;

(********************************)
(*	Load the Room from Disk		*)
(********************************)

procedure	TLevel.Load(ID:Integer);
var f:File of Byte;
	x,y:Integer;
	c:Byte;
begin
	LevelID := ID;
	NewFile := True;
	Assign(f,FileName);
	{$I-}
	Reset(f);
	{$I+}
	If IOResult = 0 then
		begin
			LevelID := ID;
			NewFile := False;
			for x := 0 to 31 do for y := 0 to 23 do Read(f,Map[x,y]);
			Close(f);
		end;
end;

(************************)
(*	Save file to disk	*)
(************************)

procedure	TLevel.Save;
var f:File of Byte;
	x,y:Integer;
	c:Byte;
begin
	Assign(f,FileName);
	Rewrite(f);
	for x := 0 to 31 do for y := 0 to 23 do Write(f,Map[x,y]);
	Close(f);
end;

(********************************)
(*	Get filename for this room	*)
(********************************)

function 	TLevel.FileName:String;
var s:String;
begin
	Str(LevelID,s);
	FileName := Concat('LEVEL',s,'.DAT');
end;

(****************************************)
(*	Initialise a tile graphic object	*)
(****************************************)

constructor	TTileGraphic.Init;
var i:Integer;
begin
	for i := 0 to 7 do Pixels[i] := 0;
end;

(****************************)
(*	Set a pixel on or off	*)
(****************************)

procedure TTileGraphic.SetPixel(x,y:Integer;On:Boolean);
begin
	x := (128 shr x);
	if On then
		Pixels[y] := Pixels[y] or x
	else
		Pixels[y] := Pixels[y] and (255 - x);
end;

(************************)
(*	Get a pixel value	*)
(************************)

function  TTileGraphic.GetPixel(x,y:Integer):Boolean;
begin
	GetPixel := (Pixels[y] and (128 shr x)) <> 0;
end;

(********************************************)
(*	Read and write the tile graphic data 	*)
(********************************************)

procedure   TTileGraphic.Read(var f:TByteFile);
var i:Integer;
begin
	for i := 0 to 7 do Pixels[i] := 0;
	if not eof(f) then
			for i := 0 to 7 do System.Read(f,Pixels[i]);
end;

procedure	TTileGraphic.Write(var f:TByteFile);
var i:Integer;
begin
	for i := 0 to 7 do System.Write(f,Pixels[i]);
end;

(********************************************)
(*	Display a tile graphic on the screen	*)
(********************************************)

procedure	TTileGraphic.Draw(x,y,Size,FCol:Integer);
var x1,y1,xc,yc,c:Integer;
begin
	if Size <= 4 then
		begin
			SetFillStyle(SolidFill,Black);
			Bar(x,y,x+8*Size-1,y+8*Size-1);
		end;

	for x1 := 0 to 7 do
		for y1 := 0 to 7 do
			if Size > 4 then
				begin
					xc := x + x1 * Size;yc := y + y1 * Size;
					SetColor(FCol);
					if not GetPixel(x1,y1) then SetColor(Black);
					Line(xc,yc,xc+Size-1,yc);
					Line(xc,yc,xc,yc+Size-1);
					SetColor(FCol);
					if GetPixel(x1,y1) then SetColor(Black);
					Line(xc+Size-1,yc+Size-1,xc,yc+Size-1);
					Line(xc+Size-1,yc+Size-1,xc+Size-1,yc);
				end
			else
				begin
                for xc := 0 to Size-1 do
					for yc := 0 to Size-1 do
						if GetPixel(x1,y1) then
							PutPixel(x+x1*Size+xc,y+y1*Size+yc,FCol);
				end;
end;

(********************************)
(* 	64 element tile set object	*)
(********************************)

constructor TTileSet.Init;
var i:Integer;
	f:TByteFile;
begin
	Assign(f,'GRAPH.BIN');
	Reset(f);
	for i := 0 to TileSetSize-1 do Tile[i].Init;
	for i := 0 to TileSetSize-1 do Tile[i].Read(f);
	Close(f);
end;

destructor  TTileSet.Done;
var i:Integer;
	f:TByteFile;
begin
end;

begin
end.