(************************************************************************)
(*																		*)
(*								GUI Objects								*)
(*																		*)
(************************************************************************)

unit GuiObj;

interface

uses TClass;

type TGuiObject = object				{ Base class }
	OrgX,OrgY,Left,Top,Right,Bottom:Integer;
	constructor Init(x,y,Width,Height:Integer);
	destructor  Done;
	procedure	Click(x,y:Integer);virtual;
	procedure	Paint;virtual;
	function	InRect(x,y:Integer):Boolean;
end;

type TText = object(TGuiObject)			{ Text object class }
	OText:String;
	constructor	Init(x,y:Integer;s:String);
	procedure	Paint;virtual;
end;

type TNumeric = object(TGuiObject)		{ Basic numeric control }
	Value,Min,Max:Integer;
	constructor	Init(x1,y1,w,h,Low,High:Integer);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure	Put(n:Integer);
	function	Get:Integer;
end;

type TCheckBox = object(TGuiObject)		{ Checkbox control }
	IsOn:Boolean;
	constructor	Init(x1,y1:Integer;Start:Boolean);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure	Put(On:Boolean);
	function	Get:Boolean;
end;

type TTileSelector = object(TGuiObject)
	TSize:Integer;
	Selected:Integer;
	Tiles:PTileSet;
	constructor Init(x,y:Integer;TS:PTileSet);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure   Refresh(n:Integer);
end;

type PGuiObject = ^TGuiObject;
	 PText = ^TText;
	 PNumeric = ^TNumeric;
	 PCheckBox = ^TCheckBox;
	 PTileSelector = ^TTileSelector;

const MaxGroupSize = 32;

type TGroup = object
	Count:Integer;
	Obj:array[1..MaxGroupSize] of PGuiObject;
	constructor Init;
	destructor	Done;
	procedure	Paint;
	procedure   Add(p:PGuiObject);
	function	Click(x,y:Integer):Integer;
end;

type PGroup = ^TGroup;

var	 ActionButton:Integer;

implementation

uses Video,Graph;

(*********************************************************************)
(*	Base GUI Object which is a class from which others are derived	 *)
(*********************************************************************)

constructor TGuiObject.Init(x,y,Width,Height:Integer);
begin
	OrgX := x;OrgY := y;
	Left := x;Top := y - Height div 2;
	Right := x + Width-1;Bottom := Top+Height-1;
end;

destructor  TGuiObject.Done;
begin end;

procedure	TGuiObject.Click(x,y:Integer);
begin end;

procedure	TGuiObject.Paint;
begin end;

function	TGuiObject.InRect(x,y:Integer):Boolean;
begin
	InRect := (x >= Left) and (x <= Right) and (y >= Top) and (y <= Bottom);
end;

(****************************)
(*	Basic text item object	*)
(****************************)

constructor	TText.Init(x,y:Integer;s:String);
begin
	OText := s;
	TGuiObject.Init(x,y,TextWidth(s),TextHeight(s));
end;

procedure	TText.Paint;
begin
	SetColor(Black);OutTextXY(Left,Top,OText);
end;

(******************************)
(*	Numeric selector object	  *)
(******************************)

constructor	TNumeric.Init(x1,y1,w,h,Low,High:Integer);
begin
	Value := Low;Min := Low;Max := High;
	TGuiObject.Init(x1,y1,w,h);
end;

procedure	TNumeric.Put(n:Integer);
begin
	if n > Max then n := Max;
	if n < Min then n := Min;
	Value := n;Paint;
end;

function	TNumeric.Get:Integer;
begin
	Get := Value;
end;

procedure	TNumeric.Click(x,y:Integer);
var n,w,Border:Integer;
begin
	Border := (Right - Left) div 7;
	n := Get;w := Right-Left;
	if x < Border then n := n - 9;
	if x < Border*2 then Dec(n);
	if x > w-Border*2 then Inc(n);
	if x > w-Border then n := n + 9;
	if n <> Get then Put(n);
end;

procedure	TNumeric.Paint;
var Border:Integer;
	s:String;
begin
	Box3D(Left,Top,Right,Bottom,True);
	Border := (Right - Left) div 7;
	Box3D(Left+1,Top+1,Left+Border,Bottom-1,False);
	Box3D(Left+Border+1,Top+1,Left+Border*2,Bottom-1,False);
	Box3D(Right-Border,Top+1,Right-1,Bottom-1,False);
	Box3D(Right-Border*2,Top+1,Right-Border-1,Bottom-1,False);
	Str(Value,s);
	OutTextXY((Left+Right) div 2 - TextWidth(s) div 2+1,
			  (Top+Bottom) div 2 - TextHeight(s) div 2+1,s);
end;

(************************************)
(*	Simple 2-state checkbox object	*)
(************************************)

constructor	TCheckBox.Init(x1,y1:Integer;Start:Boolean);
begin
	IsOn := Start;
	TGuiObject.Init(x1,y1,10,10);
end;

procedure	TCheckBox.Paint;
begin
	Box3D(Left,Top,Right,Bottom,True);
	if IsOn then
		begin
			SetFillStyle(SolidFill,Black);
			Bar(Left+2,Top+2,Right-3,Bottom-3);
		end;
end;

procedure	TCheckBox.Click(x,y:Integer);
begin
	Put(not Get);
end;

procedure	TCheckBox.Put(On:Boolean);
begin
	if On <> IsOn then
		begin
			IsOn := On;Paint;
		end;
end;

function	TCheckBox.Get:Boolean;
begin
	Get := IsOn;
end;

(********************)
(*	Tile selector	*)
(********************)

constructor TTileSelector.Init(x,y:Integer;TS:PTileSet);
begin
	Tiles := TS;TSize := 2;Selected := 0;
	TGuiObject.Init(x,y,4*10*TSize+2,4*10*TSize+2);
end;

procedure   TTileSelector.Refresh(n:Integer);
var x1,y1,c:Integer;
begin
	c := Green;if n = Selected then c := Yellow;
	x1 := Left + n mod 4 * 10*TSize + 3;
	y1 := Top + n div 4 * 10*TSize + 3;
	Tiles^.Tile[n].Draw(x1,y1,TSize,c);
	c := LightGray;if n = Selected then c := White;
	SetColor(c);Rectangle(x1-1,y1-1,x1+8*TSize,y1+8*TSize);
end;

procedure	TTileSelector.Click(x,y:Integer);
begin
	x := (x - 3) div (10*TSize);
	y := (y - 3) div (10*TSize);
	if x > 3 then x := 3;
	x := x + y * 4;
	if x >= TileSetSize then x := TileSetSize-1;
	if x <> Selected then
		begin
			y := Selected;Selected := -1;Refresh(y);
			Selected := x;Refresh(Selected);
		end;
end;

procedure	TTileSelector.Paint;
var i:Integer;
begin
	Box3D(Left,Top,Right,Bottom,False);
	for i := 0 to 15 do Refresh(i);
end;

(**************************)
(*	Grouped object type	  *)
(**************************)

constructor TGroup.Init;
begin
	Count := 0;
end;

destructor	TGroup.Done;
var i:Integer;
begin
	for i := 1 to Count do Dispose(Obj[i],Done);
	Count := 0;
end;

procedure	TGroup.Paint;
var i:Integer;
begin
	for i := 1 to Count do Obj[i]^.Paint;
end;

function	TGroup.Click(x,y:Integer):Integer;
var i:Integer;
begin
	ActionButton := 0;
	for i := 1 to Count do
		if Obj[i]^.InRect(x,y) then
				Obj[i]^.Click(x-Obj[i]^.Left,y-Obj[i]^.Top);
	Click := ActionButton;
end;

procedure   TGroup.Add(p:PGuiObject);
begin
	Inc(Count);
	Obj[Count] := p;
end;

begin
end.
